#include "libgccvb.h"
#include "aligngfx.h"
#include "title.h"
#include "flappy.h"

const signed char flap_mv[] = {3,3,3,3,3,3,2,2,2,2,2,1,1,1,1,1,0,0,0,0,0,-1,-1,-1,-1,-2,-2,-2,-2,-3,-3,-3,-4,-4,-4,-5,-5,-6,-6,-7,-8,-9,-10,-11,-12,-13,-14,-15,-16,-18,-20,-22};

#define MAX_FLAP (sizeof(flap_mv)-1)

//pipe modes
#define NORMAL 2
#define MOVE_UP 0
#define MOVE_DOWN 1
#define SQUEEZE_TU_BU 4 //top up, bottom up
#define SQUEEZE_TU_BD 5 //top up, bottom down
#define SQUEEZE_TD_BU 6 //top down, bottom up
#define SQUEEZE_TD_BD 7 //top down, bottom down

#define WRLD_SCORE 21
#define WRLD_PIPE3BOTTOM 22
#define WRLD_PIPE3TOP 23
#define WRLD_PIPE2BOTTOM 24
#define WRLD_PIPE2TOP 25
#define WRLD_PIPE1BOTTOM 26
#define WRLD_PIPE1TOP 27
#define WRLD_CHEEP 28
#define WRLD_FOREGND 29
#define WRLD_MIDGND 30
#define WRLD_BKGND 31
#define WRLD_LAST 20

void show_alignment()
{
	int i;

	copymem((void*) (CharSeg0), (void*) alignset, 128 * 16); // Copy the chars to CharMap 0
	copymem((void*) BGMap(0), (void*) alignmap, 1024 * 8);  // Copy the char index to BGMap 0

	vbSetWorld(31, WRLD_ON, 144, 3, 64, 0, 0, 0, 94, 94);
	vbSetWorld(30, WRLD_ON, 120, 0, 80, 0, 0, 95, 143, 38);
	vbSetWorld(29, WRLD_ON, 136, -3, 91, 0, 0, 134, 119, 22);
	vbSetWorld(28, WRLD_LON, 0, 0, 0, 0, 0, 157, 16, 14);
	vbSetWorld(27, WRLD_LON, 368, 0, 210, 0, 0, 157, 16, 14);
	vbSetWorld(26, WRLD_RON, 368, 0, 0, 0, 0, 157, 16, 14);
	vbSetWorld(25, WRLD_RON, 0, 0, 210, 0, 0, 157, 16, 14);
	vbSetWorld(24, WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);
	vbFXFadeIn(10);
	while(!(vbReadPad()&(K_STA|K_A)));
	vbFXFadeOut(10);
}

//title screen
void title_screen()
{
    int brightness, wait, i;

    //SETUP
    copymem((void*)CharSeg0, (void*)chtitle, 351*16);
    copymem((void*)BGMap(0), (void*)bgtitle, 1024*8);

    vbSetWorld(31, WRLD_ON, 0, 0, 0, 0, 0, 0, 384, 42); //name
    vbSetWorld(30, WRLD_ON, 112, -10, 44, 112, 0, 44, 148, 148); //cheep cheep
    vbSetWorld(29, WRLD_ON, 100, -5, 190, 100, 0, 190, 180, 20); //press start
    vbSetWorld(28, WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0);

    //FADE IN
    brightness = 0;
    while(brightness != 32)
    {
        for (i = 0; i <= 40; i++) 
        {
            wait = VIP_REGS[XPSTTS];
            while (wait == VIP_REGS[XPSTTS]);
        }
        VIP_REGS[BRTA] = brightness;
        VIP_REGS[BRTB] = brightness*2;
        VIP_REGS[BRTC] = brightness;
        brightness++;
    } 

    //WAIT FOR USER INPUT
    while (!(vbReadPad()&(K_STA)));
  
    //FADE OUT
    brightness = 32;
    while(brightness != 0)
    {
        for (i = 0; i <= 40; i++) 
        {
            wait = VIP_REGS[XPSTTS];
            while (wait == VIP_REGS[XPSTTS]);
        }
        VIP_REGS[BRTA] = brightness;
        VIP_REGS[BRTB] = brightness*2;
        VIP_REGS[BRTC] = brightness;
        brightness--;
    } 
}

int main()
{
	int scrollcnt;
	int x=192;
	int y=112;
	int score;
	int pipe1top, pipe1bottom, pipe1mode, pipe2top, pipe2bottom, pipe2mode, pipe3top, pipe3bottom, pipe3mode;
	int flapped, flapcnt;
	int firsttime;
	int keypress;
	unsigned char randnum=0;
	
  vbDisplayOn();

	show_alignment();

  title_screen();

	copymem((void*)(CharSeg3+(384*16)), (void*)chAsciiMap, 128*16); //Copy chars to Char segment 3
	copymem((void*) CharSeg0, (void*) chflappy, 123*16); // Copy the chars to CharSeg0
	copymem((void*)BGMap(0), (void*) bgflappy, 1024*8);
	sethword((void*)BGMap(4), (2048-128), 8192/2); //clear screen w/ blank char
	
	while (1)
	{
	  scrollcnt=10;
    x = 192;
    y = 112;
    score=0;
    flapped=0;
    flapcnt=0;
    firsttime=1;

    //initial pipe locations
    pipe1top=50;
    pipe1bottom=150;
    pipe1mode=NORMAL;
    pipe2top=70;
    pipe2bottom=170;
    pipe2mode=NORMAL;
    pipe3top=90;
    pipe3bottom=190;
    pipe3mode=NORMAL;
    
    vbDisplayHide();
    
    //vbSetWorld (World, Header, Screen X, Screen P, Screen Y, Buffer X, Buffer P, Buffer Y, Width, Height)
    vbSetWorld(WRLD_SCORE, WRLD_ON|4, 0, -10, 0, 0, 0, 0, 383, 223); //score
    vbSetWorld(WRLD_CHEEP, WRLD_ON, x, -5, y, 64, 0, 0, 15, 15); //cheep cheep
    vbSetWorld(WRLD_PIPE1TOP, WRLD_ON, 128-(scrollcnt<<1), -5, 0, 32, 0, 224-pipe1top, 31, pipe1top-1); //Top Pipe 1
    vbSetWorld(WRLD_PIPE1BOTTOM, WRLD_ON, 128-(scrollcnt<<1), -5, pipe1bottom-1, 0, 0, 0, 31, 208-pipe1bottom-1); //Bottom Pipe 1
    vbSetWorld(WRLD_PIPE2TOP, WRLD_ON, 288-(scrollcnt<<1), -5, 0, 32, 0, 224-pipe2top, 31, pipe2top-1); //Top Pipe 2
    vbSetWorld(WRLD_PIPE2BOTTOM, WRLD_ON, 288-(scrollcnt<<1), -5, pipe2bottom-1, 0, 0, 0, 31, 208-pipe2bottom-1); //Bottom Pipe 2
    vbSetWorld(WRLD_PIPE3TOP, WRLD_ON, 448-(scrollcnt<<1), -5, 0, 32, 0, 224-pipe3top, 31, pipe3top-1); //Top Pipe 3
    vbSetWorld(WRLD_PIPE3BOTTOM, WRLD_ON, 448-(scrollcnt<<1), -5, pipe3bottom-1, 0, 0, 0, 31, 208-pipe3bottom-1); //Bottom Pipe 3
    vbSetWorld(WRLD_FOREGND, WRLD_ON, 0, -5, 208, 0, 0, 432, 383, 15); //foreground
    vbSetWorld(WRLD_MIDGND, WRLD_ON, 0, 0, 172, 0, 0, 396, 383, 35); //midground
    vbSetWorld(WRLD_BKGND, WRLD_ON, 0, 10, 0, 0, 0, 224, 383, 127); //background
    vbSetWorld(WRLD_LAST, WRLD_END, 0, 0, 0, 0, 0, 0, 0, 0); // Blank world and END bit set 
    
    cls(4,122);
    bgmap_print(4, 33, 2, "Score: ", 0);
    bgmap_print(4, 40, 2, itoa(score,10,1), 0);
		
		vbDisplayShow();
    
    while (1)
    {
      if (firsttime)
      {
        bgmap_print(4, 17, 20, "Press A to flap", 0);
        while (!(vbReadPad()&K_A));
        bgmap_print_blank(4, 17, 20, 15);
        firsttime=0;
      }
        
      keypress=vbReadPad();
      if (keypress & K_A)
      {
        if (flapped==0) //flap
        {
          flapped=1;
          flapcnt=0;
          randnum^=0xFF;
        }
      }
      else
      {
        flapped=0;
      }

      if ((flapcnt<12)&&(!(flapcnt&1))) //flap wings
        WA[WRLD_CHEEP].mx=80;
      else
        WA[WRLD_CHEEP].mx=64;
      
      y-=flap_mv[flapcnt];
      WA[WRLD_CHEEP].gy=y;
      randnum+=y;
      
      flapcnt++;
      if (flapcnt>MAX_FLAP)
        flapcnt=MAX_FLAP;
    
      scrollcnt++;
      if (scrollcnt>10240) //number evenly divisible by 512 and 160
        scrollcnt=0;

      //scrolling background and foreground
      WA[WRLD_FOREGND].mx=(scrollcnt<<1)&0x1FF;
      WA[WRLD_MIDGND].mx=scrollcnt&0x1FF;
      WA[WRLD_BKGND].mx=(scrollcnt>>2)&0x1FF;

      if (((scrollcnt<<1)%160)==128) //score
      {
        score++;
        bgmap_print(4, 40, 2, itoa(score,10,1), 0);
      }
      
      //update pipes
      if (((scrollcnt<<1)%160)==0) //generate and hand off pipes
      {        
        pipe1top=pipe2top;
        pipe1bottom=pipe2bottom;
        pipe1mode=pipe2mode;
        pipe2top=pipe3top;
        pipe2bottom=pipe3bottom;
        pipe2mode=pipe3mode;
        
        //generate pipes to get more difficult
        if (score<5) //easy, wide open pipes
        {
          pipe3top=(randnum&0x3F)+16; //make top between 16 and 80
          pipe3bottom=pipe3top+90;
          pipe3mode=NORMAL;
        }
        else if (score<10) //narrower
        {
          if (randnum&0x40)
          {
            pipe3top=(randnum&0x3F)+16; //make top between 16 and 80
            pipe3bottom=pipe3top+(90-(score<<1));
          }
          else
          {
            pipe3bottom=208-((randnum&0x3F)+16); //make bottom between 192 and 128
            pipe3top=pipe3bottom-(90-(score<<1));
          }
          pipe3mode=NORMAL;
        }
        else if (score<20) //moving
        {
          if (randnum&0x40)
          {
            pipe3top=(randnum&0x3F)+16; //make top between 16 and 80
            pipe3bottom=pipe3top+70;
            pipe3mode=MOVE_DOWN|(1<<4);
          }
          else
          {
            pipe3bottom=208-((randnum&0x3F)+16); //make bottom between 192 and 128
            pipe3top=pipe3bottom-70;
            pipe3mode=MOVE_UP|(1<<4);
          }
        }
        else if (score<25) //squeezing and moving
        {
          if (randnum&0x40)
          {
            pipe3top=(randnum&0x3F)+16; //make top between 16 and 80
            pipe3bottom=pipe3top+70;
            pipe3mode=SQUEEZE_TU_BU|(randnum&0x03)|(1<<4); //randomize squeeze direction too
          }
          else
          {
            pipe3bottom=208-((randnum&0x3F)+16); //make bottom between 192 and 128
            pipe3top=pipe3bottom-70;
            pipe3mode=SQUEEZE_TU_BU|(randnum&0x03)|(1<<4); //randomize squeeze direction too
          }
        }
        else //squeezing and moving faster
        {
          if (randnum&0x40)
          {
            pipe3top=(randnum&0x3F)+16; //make top between 16 and 80
            pipe3bottom=pipe3top+70;
            pipe3mode=SQUEEZE_TU_BU|(randnum&0x03)|(((randnum>>2)&0x03)<<4); //random speed (0-3)
          }
          else
          {
            pipe3bottom=208-((randnum&0x3F)+16); //make bottom between 192 and 128
            pipe3top=pipe3bottom-70;
            pipe3mode=SQUEEZE_TU_BU|(randnum&0x03)|(((randnum>>2)&0x03)<<4); //random speed (0-3)
          }
        }
      }
      
      if ((pipe1mode&0x0F)==MOVE_UP)
      {
        pipe1top-=((pipe1mode>>4)&0x03);
        pipe1bottom-=((pipe1mode>>4)&0x03);
        if (pipe1top<16)
          pipe1mode=(pipe1mode&0xF0)|MOVE_DOWN;
      }
      else if ((pipe1mode&0x0F)==MOVE_DOWN)
      {
        pipe1top+=((pipe1mode>>4)&0x03);
        pipe1bottom+=((pipe1mode>>4)&0x03);
        if (pipe1bottom>192)
          pipe1mode=(pipe1mode&0xF0)|MOVE_UP;
      }
      else if ((pipe1mode&0x0F)>=SQUEEZE_TU_BU) //squeeze mode
      {
        if ((pipe1bottom-pipe1top)<60) //too close, reverse directions
          pipe1mode^=0x03;
        if ((pipe1mode>>1)&MOVE_DOWN) //top down bit
          pipe1top+=((pipe1mode>>4)&0x03);
        else
        {
          pipe1top-=((pipe1mode>>4)&0x03);
          if (pipe1top<16)
            pipe1mode=(pipe1mode&0xFD)|(MOVE_DOWN<<1);
        }
          
        if (pipe1mode&MOVE_DOWN) //bottom down bit
        {
          pipe1bottom+=((pipe1mode>>4)&0x03);
          if (pipe1bottom>192)
            pipe1mode=(pipe1mode&0xFE)|MOVE_UP;
        }
        else
          pipe1bottom-=((pipe1mode>>4)&0x03);
      }
      
      if ((pipe2mode&0x0F)==MOVE_UP)
      {
        pipe2top-=((pipe2mode>>4)&0x03);
        pipe2bottom-=((pipe2mode>>4)&0x03);
        if (pipe2top<16)
          pipe2mode=(pipe2mode&0xF0)|MOVE_DOWN;
      }
      else if ((pipe2mode&0x0F)==MOVE_DOWN)
      {
        pipe2top+=((pipe2mode>>4)&0x03);
        pipe2bottom+=((pipe2mode>>4)&0x03);
        if (pipe2bottom>192)
          pipe2mode=(pipe2mode&0xF0)|MOVE_UP;
      }
      else if ((pipe2mode&0x0F)>=SQUEEZE_TU_BU) //squeeze mode
      {
        if ((pipe2bottom-pipe2top)<60) //too close, reverse directions
          pipe2mode^=0x03;
        if ((pipe2mode>>1)&MOVE_DOWN) //top down bit
          pipe2top+=((pipe2mode>>4)&0x03);
        else
        {
          pipe2top-=((pipe2mode>>4)&0x03);
          if (pipe2top<16)
            pipe2mode=(pipe2mode&0xFD)|(MOVE_DOWN<<1);
        }
          
        if (pipe2mode&MOVE_DOWN) //bottom down bit
        {
          pipe2bottom+=((pipe2mode>>4)&0x03);
          if (pipe2bottom>192)
            pipe2mode=(pipe2mode&0xFE)|MOVE_UP;
        }
        else
          pipe2bottom-=((pipe2mode>>4)&0x03);
      }
      
      if ((pipe3mode&0x0F)==MOVE_UP)
      {
        pipe3top-=((pipe3mode>>4)&0x03);
        pipe3bottom-=((pipe3mode>>4)&0x03);
        if (pipe3top<16)
          pipe3mode=(pipe3mode&0xF0)|MOVE_DOWN;
      }
      else if ((pipe3mode&0x0F)==MOVE_DOWN)
      {
        pipe3top+=((pipe3mode>>4)&0x03);
        pipe3bottom+=((pipe3mode>>4)&0x03);
        if (pipe3bottom>192)
          pipe3mode=(pipe3mode&0xF0)|MOVE_UP;
      }
      else if ((pipe3mode&0x0F)>=SQUEEZE_TU_BU) //squeeze mode
      {
        if ((pipe3bottom-pipe3top)<60) //too close, reverse directions
          pipe3mode^=0x03;
        if ((pipe3mode>>1)&MOVE_DOWN) //top down bit
          pipe3top+=((pipe3mode>>4)&0x03);
        else
        {
          pipe3top-=((pipe3mode>>4)&0x03);
          if (pipe3top<16)
            pipe3mode=(pipe3mode&0xFD)|(MOVE_DOWN<<1);
        }
          
        if (pipe3mode&MOVE_DOWN) //bottom down bit
        {
          pipe3bottom+=((pipe3mode>>4)&0x03);
          if (pipe3bottom>192)
            pipe3mode=(pipe3mode&0xFE)|MOVE_UP;
        }
        else
          pipe3bottom-=((pipe3mode>>4)&0x03);
      }

      WA[WRLD_PIPE1TOP].h=pipe1top-1;
      WA[WRLD_PIPE1TOP].gx=160-((scrollcnt<<1)%160)-32;
      WA[WRLD_PIPE1TOP].my=224-pipe1top;

      WA[WRLD_PIPE1BOTTOM].h=208-pipe1bottom-1;
      WA[WRLD_PIPE1BOTTOM].gx=160-((scrollcnt<<1)%160)-32;
      WA[WRLD_PIPE1BOTTOM].gy=pipe1bottom-1;
      
      WA[WRLD_PIPE2TOP].h=pipe2top-1;
      WA[WRLD_PIPE2TOP].gx=(160-((scrollcnt<<1)%160))+160-32;
      WA[WRLD_PIPE2TOP].my=224-pipe2top;

      WA[WRLD_PIPE2BOTTOM].h=208-pipe2bottom-1;
      WA[WRLD_PIPE2BOTTOM].gx=(160-((scrollcnt<<1)%160))+160-32;
      WA[WRLD_PIPE2BOTTOM].gy=pipe2bottom-1;

      WA[WRLD_PIPE3TOP].h=pipe3top-1;
      WA[WRLD_PIPE3TOP].gx=(160-((scrollcnt<<1)%160))+320-32;
      WA[WRLD_PIPE3TOP].my=224-pipe3top;

      WA[WRLD_PIPE3BOTTOM].h=208-pipe3bottom-1;
      WA[WRLD_PIPE3BOTTOM].gx=(160-((scrollcnt<<1)%160))+320-32;
      WA[WRLD_PIPE3BOTTOM].gy=pipe3bottom-1;
      
      if ((y>192)||((((scrollcnt<<1)%160)>=80)&&(((scrollcnt<<1)%160)<=127))&&((y<pipe2top)||(y>(pipe2bottom-16)))) //die
      {
          vbSetWorld(31, WRLD_ON|4, 0, 0, 0, 0, 0, 0, 383, 223); //score
          WA[30].head=WRLD_END;
          bgmap_print_blank(4, 33, 2, 12);
          bgmap_print(4, 19, 14, "Score: ", 0);
          bgmap_print(4, 26, 14, itoa(score,10,1), 0);
          bgmap_print(4, 12, 20, "Press Start to play again", 0);
          while (!(vbReadPad() & K_STA));
          break;
      }
      
      while (!(VIP_REGS[XPSTTS] & XPBSYR));
      while (VIP_REGS[XPSTTS] & XPBSYR);
    }
  }
	return 0;
}
